import { getIsSameDate,
  getIsNextDate,
  getIsPreviousDate, 
  getIsBetweenStartingAndEndingDate } from '../../../../../../utils';
import { DateCategory } from '../../../calendarModel';
import { GetDateCategories } from './getDateCategoriesModel';

export const getDateCategories: GetDateCategories = ({
  currentDate,
  dateMonthAndYear,
  disabledDaysOfTheWeek,
  disabledDatesNumberOfTheMonth: disabledDatesOfTheMonth,
  specialDatesOfTheMonth,
  weekDayIndex,
  startingDate,
  endingDate,
  selectedDate = null,
}) => {
  const dateCategories: DateCategory[] = [];

  getIsSameDate(currentDate, dateMonthAndYear)
    && dateCategories.push(DateCategory.NOW);

  selectedDate
    && getIsSameDate(selectedDate, dateMonthAndYear)
    && dateCategories.push(DateCategory.SELECTED);

  startingDate && endingDate
    && !getIsBetweenStartingAndEndingDate({ startingDate, testDate: dateMonthAndYear, endingDate })
    && dateCategories.push(DateCategory.DISABLED);

  startingDate && !endingDate
    && getIsPreviousDate(dateMonthAndYear, startingDate)
    && dateCategories.push(DateCategory.DISABLED);

  endingDate && !startingDate
    && getIsNextDate(dateMonthAndYear, endingDate)
    && dateCategories.push(DateCategory.DISABLED);

  disabledDaysOfTheWeek
    && disabledDaysOfTheWeek.some(
      disabledIndexDay => weekDayIndex === disabledIndexDay,
    )
    && dateCategories.push(DateCategory.DISABLED);

  disabledDatesOfTheMonth
    && disabledDatesOfTheMonth.some(
      ({ datesNumber: dates, monthNumber: month, year }) => dateMonthAndYear.monthNumber === month 
        && dateMonthAndYear.year === year 
        && dates.some(date => dateMonthAndYear.dateNumber === date))
    && dateCategories.push(DateCategory.DISABLED);

  specialDatesOfTheMonth
    && specialDatesOfTheMonth.some(
      ({ datesNumber: dates, monthNumber: month, year }) => dateMonthAndYear.monthNumber === month 
        && dateMonthAndYear.year === year 
        && dates.some(date => dateMonthAndYear.dateNumber === date))
    && dateCategories.push(DateCategory.SPECIAL);

  return dateCategories;
};
