import { getDatesOfSelectedMonthList, getWeekDayIndex } from '../../../../../utils';
import { getDateCategories } from './getDateCategories';
import { DateOfTheMonthData, DateCategory } from '../../calendarModel';
import { SetDisabledDatesList, 
  FillDateCategories, 
  SetEnabledDatesList, 
  GetDatesOfMonthData } from './getDatesOfMonthDataModel';

const setDisabledDatesList: SetDisabledDatesList = (dates) => {
  return dates.map(({ dateNumber }) => ({
    dateNumber,
    dateCategories: [DateCategory.DISABLED],
    key: `${dateNumber}-${DateCategory.DISABLED}`,
  })) as DateOfTheMonthData[];
};

const setEnabledDatesList: SetEnabledDatesList = (dates, fillDatesCategories) => {
  return dates.map(({ dateNumber, monthNumber, year }) => {
    const weekDayIndex = getWeekDayIndex({ dateNumber, monthNumber, year });
    return {
      dateNumber,
      dateCategories: fillDatesCategories(
        { dateNumber, monthNumber, year },
        weekDayIndex,
      ),
      key: `${dateNumber}`,
    };
  });
};

export const getDatesOfMonthData: GetDatesOfMonthData = ({ 
  monthNumber,
  year,
  selectedDate,
  ...categoriesParams
}) => {
  const {
    datesOfPreviousMonthList,
    datesOfCurrentMonthList,
    datesOfNextMonthList,
  } = getDatesOfSelectedMonthList(monthNumber, year);

  const previousDatesDisabled = setDisabledDatesList(datesOfPreviousMonthList);
  const nextDatesDisabled = setDisabledDatesList(datesOfNextMonthList);

  const fillDayCategories: FillDateCategories = (
    enabledDate, weekDayIndex,
  ) => getDateCategories({
    dateMonthAndYear: enabledDate,
    weekDayIndex,
    selectedDate,
    ...categoriesParams,
  });

  const currentDates = setEnabledDatesList(datesOfCurrentMonthList, fillDayCategories);

  return [...previousDatesDisabled, ...currentDates, ...nextDatesDisabled];
};
