import React, { FC } from 'react';
import { View } from 'react-native';
import { ArrowDownIcon, ArrowUpIcon, Body2RegularText, LabelMediumText, LabelRegularText, PrimaryExtraLargeIconFlatButton, SecondaryExtraLargeIconFlatButton, StopIcon } from '../../../../../atoms';
import { ShutterLoadingInfoTextModel } from '../../../widgetDetailModels';
import { useShutter } from '../../hooks/useShutter';
import { ShutterAction, ShutterControlPanelModel, ShutterControlPanelValue, ShutterOngoingAction } from '../shutterModels';
import { styles } from '../styles';

const shutterControlPanelValues: { [key: string]: ShutterControlPanelValue } = {
  DOWN: 'DOWN',
  STOP: 'STOP',
  UP: 'UP',
};

const ongoingActionTypes: { [key: string]: ShutterAction } = {
  OPEN: 'OPEN',
  UP: 'UP',
  CLOSE: 'CLOSE',
  DOWN: 'DOWN',
  STOP: 'STOP',
};

const getOngoingActionText = (
  { ongoingAction, textOptions }: {ongoingAction: ShutterOngoingAction, textOptions: ShutterLoadingInfoTextModel}) => {
  const ongoingActionTextByActionType = {
    [ongoingActionTypes.OPEN]: textOptions?.opening,
    [ongoingActionTypes.UP]: textOptions?.opening,
    [ongoingActionTypes.CLOSE]: textOptions?.closing,
    [ongoingActionTypes.DOWN]: textOptions?.closing,
    [ongoingActionTypes.STOP]: textOptions?.stopping,
  };
  return ongoingAction && ongoingActionTextByActionType[ongoingAction];
};

export const ShutterControlPanel: FC<ShutterControlPanelModel> = ({
  title,
  widgetName,
  widgetStateInfoOptions,
  loadingInfoTextOptions,
  value,
  ongoingAction,
  onClick,
  loading,
}) => {
  const { shutterAbsoluteAperture } = useShutter(value);
  const isOpenButtonDisabled: boolean = loading || shutterAbsoluteAperture.FULLY_OPENED;
  const isCloseButtonDisabled: boolean = loading || shutterAbsoluteAperture.FULLY_CLOSED;
  const isStopButtonDisabled: boolean = loading || 
    shutterAbsoluteAperture.FULLY_OPENED || 
    shutterAbsoluteAperture.FULLY_CLOSED;
  
  return (
    <View style={styles.controlPanelWrapper}>
      <View style={styles.controlPanelTitle}>
        <LabelRegularText numberOfLines={1} color={styles.controlPanelTitle.color}>{title}</LabelRegularText>
      </View>
      <View>
        <View style={styles.controlPanelInfo}>
          <View>
            <View style={styles.controlPanelInfoTitle}>
              <LabelMediumText numberOfLines={1} color={styles.controlPanelInfoTitle.color}>
                {widgetName}
              </LabelMediumText>
            </View>
            <View>
              {loading && ongoingAction && 
              <Body2RegularText numberOfLines={1} color={styles.controlPanelInfoSubtitle.color}>
                { 
                  loadingInfoTextOptions &&
                      getOngoingActionText({ ongoingAction, textOptions: loadingInfoTextOptions })
                }
              </Body2RegularText>
              }
              {!loading &&
                <Body2RegularText numberOfLines={1} color={styles.controlPanelInfoSubtitle.color}>
                  {value}% { widgetStateInfoOptions?.open}
                </Body2RegularText>
              }
            </View>
          </View>
        </View>
        <View style={styles.controlPanelControlsWrapper}>
          <View
            style={styles.controlPanelControls}
          >
            <PrimaryExtraLargeIconFlatButton
              Icon={ArrowDownIcon}
              disabled={isCloseButtonDisabled}
              onClick={() => onClick?.(shutterControlPanelValues.DOWN)}
            />
            <SecondaryExtraLargeIconFlatButton
              Icon={StopIcon}
              disabled={isStopButtonDisabled}
              onClick={() => onClick?.(shutterControlPanelValues.STOP)}
            />
            <PrimaryExtraLargeIconFlatButton
              Icon={ArrowUpIcon}
              disabled={isOpenButtonDisabled}
              onClick={() => onClick?.(shutterControlPanelValues.UP)}
            />
          </View>
        </View>
      </View>
    </View>
  );
};
