import React, { FC, useEffect, useState } from 'react';
import { View } from 'react-native';
import { SwitchAndDimmerWidgetDetailModel } from '../widgetDetailModels';
import { SetDimmer } from './components/setDimmer';
import { SetSwitch } from './components/setSwitch';
import { styles } from './styles';

const booleanValues = {
  MAX: 100,
  MIN: 0,
};

const valueFromBoolean = {
  true: booleanValues.MAX,
  false: booleanValues.MIN,
};

const getRoundedValue = (value: string | number): number => {
  return typeof value === 'string' ? Math.round(Number(value)) : Math.round(value);
};

export const SwitchAndDimmerWidgetDetail: FC<SwitchAndDimmerWidgetDetailModel> = ({
  title,
  subtitle,
  loadingInfoTextOptions,
  sliderTitle,
  value = booleanValues.MIN,
  id,
  onUpdateData,
  loading = false,
}) => {
  const [state, setState] = useState<number>(getRoundedValue(value));
  const [isEnabled, setIsEnabled] = useState<boolean>(!!getRoundedValue(value));
  const [lightIntensity, setLightIntensity] = useState<number>(getRoundedValue(value));

  useEffect(() => {
    setIsEnabled(value !== booleanValues.MIN);
    setState(getRoundedValue(value));
  }, [value]);

  useEffect(() => {
    onUpdateData && onUpdateData({ id, value: lightIntensity.toString() });
  }, [lightIntensity]);

  const handleOnChangeSwitchValue = (response: boolean) => {
    const editValue = valueFromBoolean[`${response}`];
    setIsEnabled(response);
    setLightIntensity(editValue);
    setState(editValue);
  };

  const handleOnFinishDimmer = (intensity: number) => {
    const editValueEnabled = !!intensity;
    setIsEnabled(editValueEnabled);
    setLightIntensity(intensity);
    setState(intensity);
  };

  return (
    <View>
      <View style={styles.setSwitchContainer}>
        <SetSwitch
          title={title}
          subtitle={subtitle}
          loadingInfoTextOptions={loadingInfoTextOptions}
          value={!!state}
          onValueChange={handleOnChangeSwitchValue}
          loading={loading}
          enabled={isEnabled}
        />
      </View>
      <SetDimmer
        defaultValue={lightIntensity}
        onFinishChange={handleOnFinishDimmer}
        loading={loading}
        sliderTitle={sliderTitle}
      />
    </View>
  );
};
